@@ -586,16 +586,13 @@ enable_toggled (GtkToggleButton *button, gpointer user_data)
 }
 
 static void
-finish_setup (CEPageDcb *self, gpointer unused, GError *error, gpointer user_data)
+finish_setup (CEPageDcb *self, gpointer user_data)
 {
 	CEPage *parent = CE_PAGE (self);
 	CEPageDcbPrivate *priv = CE_PAGE_DCB_GET_PRIVATE (self);
 	NMSettingDcb *s_dcb = nm_connection_get_setting_dcb (parent->connection);
 	guint i;
 
-	if (error)
-		return;
-
 	gtk_toggle_button_set_active (priv->enabled, priv->initial_have_dcb);
 	g_signal_connect (priv->enabled, "toggled", G_CALLBACK (enable_toggled), self);
 	gtk_widget_set_sensitive (GTK_WIDGET (priv->box), priv->initial_have_dcb);
@@ -643,7 +640,7 @@ ce_page_dcb_new (NMConnectionEditor *editor,
 	} else
 		priv->options = (NMSettingDcb *) nm_setting_dcb_new ();
 
-	g_signal_connect (self, "initialized", G_CALLBACK (finish_setup), NULL);
+	g_signal_connect (self, CE_PAGE_INITIALIZED, G_CALLBACK (finish_setup), NULL);
 
 	return CE_PAGE (self);
 }
