@@ -250,14 +250,11 @@ apn_filter_cb (GtkEditable *editable,
 }
 
 static void
-finish_setup (CEPageMobile *self, gpointer unused, GError *error, gpointer user_data)
+finish_setup (CEPageMobile *self, gpointer user_data)
 {
 	CEPage *parent = CE_PAGE (self);
 	CEPageMobilePrivate *priv = CE_PAGE_MOBILE_GET_PRIVATE (self);
 
-	if (error)
-		return;
-
 	if (NM_IS_SETTING_GSM (priv->setting))
 		populate_gsm_ui (self, parent->connection);
 	else if (NM_IS_SETTING_CDMA (priv->setting))
@@ -329,7 +326,7 @@ ce_page_mobile_new (NMConnectionEditor *editor,
 		return NULL;
 	}
 
-	g_signal_connect (self, "initialized", G_CALLBACK (finish_setup), NULL);
+	g_signal_connect (self, CE_PAGE_INITIALIZED, G_CALLBACK (finish_setup), NULL);
 
 	return CE_PAGE (self);
 }
