@@ -45,7 +45,7 @@
  * Evaluates to the micro version number of NetworkManager which this source
  * compiled against.
  */
-#define NMA_MICRO_VERSION (20)
+#define NMA_MICRO_VERSION (22)
 
 /**
  * NMA_CHECK_VERSION:
@@ -72,9 +72,10 @@
 #define NMA_VERSION_1_8_8  (NMA_ENCODE_VERSION (1, 8, 8))
 #define NMA_VERSION_1_8_10 (NMA_ENCODE_VERSION (1, 8, 10))
 #define NMA_VERSION_1_8_12 (NMA_ENCODE_VERSION (1, 8, 12))
+#define NMA_VERSION_1_8_22 (NMA_ENCODE_VERSION (1, 8, 22))
 
 #define NMA_VERSION_CUR_STABLE  NMA_VERSION_1_8_10
-#define NMA_VERSION_NEXT_STABLE NMA_VERSION_1_8_12
+#define NMA_VERSION_NEXT_STABLE NMA_VERSION_1_8_22
 
 #define NMA_VERSION NMA_ENCODE_VERSION (NMA_MAJOR_VERSION, NMA_MINOR_VERSION, NMA_MICRO_VERSION)
 
@@ -123,4 +124,10 @@
 # define NMA_AVAILABLE_IN_1_8
 #endif
 
+#if NMA_VERSION_MAX_ALLOWED < NMA_VERSION_1_8_22
+# define NMA_AVAILABLE_IN_1_8_22            G_UNAVAILABLE(1.8,22)
+#else
+# define NMA_AVAILABLE_IN_1_8_22
+#endif
+
 #endif  /* NMA_VERSION_H */
