@@ -36,17 +36,23 @@ struct _EAPMethodSimple {
 
 	WirelessSecurity *ws_parent;
 
+	const char *password_flags_name;
 	EAPMethodSimpleType type;
 	EAPMethodSimpleFlags flags;
 
+	gboolean username_requested;
+	gboolean password_requested;
+	gboolean pkey_passphrase_requested;
 	GtkEntry *username_entry;
 	GtkEntry *password_entry;
 	GtkToggleButton *show_password;
+	GtkEntry *pkey_passphrase_entry;
+	GtkToggleButton *show_pkey_passphrase;
 	guint idle_func_id;
 };
 
 static void
-show_toggled_cb (GtkToggleButton *button, EAPMethodSimple *method)
+show_password_toggled_cb (GtkToggleButton *button, EAPMethodSimple *method)
 {
 	gboolean visible;
 
@@ -54,6 +60,15 @@ show_toggled_cb (GtkToggleButton *button, EAPMethodSimple *method)
 	gtk_entry_set_visibility (method->password_entry, visible);
 }
 
+static void
+show_pkey_passphrase_toggled_cb (GtkToggleButton *button, EAPMethodSimple *method)
+{
+	gboolean visible;
+
+	visible = gtk_toggle_button_get_active (button);
+	gtk_entry_set_visibility (method->pkey_passphrase_entry, visible);
+}
+
 static gboolean
 always_ask_selected (GtkEntry *passwd_entry)
 {
@@ -68,27 +83,45 @@ validate (EAPMethod *parent, GError **error)
 	const char *text;
 	gboolean ret = TRUE;
 
-	text = gtk_entry_get_text (method->username_entry);
-	if (!text || !strlen (text)) {
-		widget_set_error (GTK_WIDGET (method->username_entry));
-		g_set_error_literal (error, NMA_ERROR, NMA_ERROR_GENERIC, _("missing EAP username"));
-		ret = FALSE;
-	} else
-		widget_unset_error (GTK_WIDGET (method->username_entry));
+	if (method->username_requested) {
+		text = gtk_entry_get_text (method->username_entry);
+		if (!text || !strlen (text)) {
+			widget_set_error (GTK_WIDGET (method->username_entry));
+			g_set_error_literal (error, NMA_ERROR, NMA_ERROR_GENERIC, _("missing EAP username"));
+			ret = FALSE;
+		} else
+			widget_unset_error (GTK_WIDGET (method->username_entry));
+	}
 
 	/* Check if the password should always be requested */
-	if (always_ask_selected (method->password_entry))
-		widget_unset_error (GTK_WIDGET (method->password_entry));
-	else {
-		text = gtk_entry_get_text (method->password_entry);
+	if (method->password_requested) {
+		if (always_ask_selected (method->password_entry))
+			widget_unset_error (GTK_WIDGET (method->password_entry));
+		else {
+			text = gtk_entry_get_text (method->password_entry);
+			if (!text || !strlen (text)) {
+				widget_set_error (GTK_WIDGET (method->password_entry));
+				if (ret) {
+					g_set_error_literal (error, NMA_ERROR, NMA_ERROR_GENERIC,
+					                     _("missing EAP password"));
+					ret = FALSE;
+				}
+			} else
+				widget_unset_error (GTK_WIDGET (method->password_entry));
+		}
+	}
+
+	if (method->pkey_passphrase_requested) {
+		text = gtk_entry_get_text (method->pkey_passphrase_entry);
 		if (!text || !strlen (text)) {
-			widget_set_error (GTK_WIDGET (method->password_entry));
+			widget_set_error (GTK_WIDGET (method->pkey_passphrase_entry));
 			if (ret) {
-				g_set_error_literal (error, NMA_ERROR, NMA_ERROR_GENERIC, _("missing EAP password"));
+				g_set_error_literal (error, NMA_ERROR, NMA_ERROR_GENERIC,
+				                     _("missing EAP client Private Key passphrase"));
 				ret = FALSE;
 			}
 		} else
-			widget_unset_error (GTK_WIDGET (method->password_entry));
+			widget_unset_error (GTK_WIDGET (method->pkey_passphrase_entry));
 	}
 
 	return ret;
@@ -97,15 +130,26 @@ validate (EAPMethod *parent, GError **error)
 static void
 add_to_size_group (EAPMethod *parent, GtkSizeGroup *group)
 {
+	EAPMethodSimple *method = (EAPMethodSimple *) parent;
 	GtkWidget *widget;
 
-	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_username_label"));
-	g_assert (widget);
-	gtk_size_group_add_widget (group, widget);
+	if (method->username_requested) {
+		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_username_label"));
+		g_assert (widget);
+		gtk_size_group_add_widget (group, widget);
+	}
 
-	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_password_label"));
-	g_assert (widget);
-	gtk_size_group_add_widget (group, widget);
+	if (method->password_requested) {
+		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_password_label"));
+		g_assert (widget);
+		gtk_size_group_add_widget (group, widget);
+	}
+
+	if (method->pkey_passphrase_requested) {
+		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_pkey_passphrase_label"));
+		g_assert (widget);
+		gtk_size_group_add_widget (group, widget);
+	}
 }
 
 typedef struct {
@@ -115,17 +159,19 @@ typedef struct {
 
 /* Indexed by EAP_METHOD_SIMPLE_TYPE_* */
 static const EapType eap_table[EAP_METHOD_SIMPLE_TYPE_LAST] = {
-	[EAP_METHOD_SIMPLE_TYPE_PAP]       = { "pap",      FALSE },
-	[EAP_METHOD_SIMPLE_TYPE_MSCHAP]    = { "mschap",   FALSE },
-	[EAP_METHOD_SIMPLE_TYPE_MSCHAP_V2] = { "mschapv2", TRUE  },
-	[EAP_METHOD_SIMPLE_TYPE_MD5]       = { "md5",      TRUE  },
-	[EAP_METHOD_SIMPLE_TYPE_PWD]       = { "pwd",      TRUE  },
-	[EAP_METHOD_SIMPLE_TYPE_CHAP]      = { "chap",     FALSE },
-	[EAP_METHOD_SIMPLE_TYPE_GTC]       = { "gtc",      TRUE  },
+	[EAP_METHOD_SIMPLE_TYPE_PAP]             = { "pap",      FALSE },
+	[EAP_METHOD_SIMPLE_TYPE_MSCHAP]          = { "mschap",   FALSE },
+	[EAP_METHOD_SIMPLE_TYPE_MSCHAP_V2]       = { "mschapv2", TRUE  },
+	[EAP_METHOD_SIMPLE_TYPE_PLAIN_MSCHAP_V2] = { "mschapv2", FALSE },
+	[EAP_METHOD_SIMPLE_TYPE_MD5]             = { "md5",      TRUE  },
+	[EAP_METHOD_SIMPLE_TYPE_PWD]             = { "pwd",      TRUE  },
+	[EAP_METHOD_SIMPLE_TYPE_CHAP]            = { "chap",     FALSE },
+	[EAP_METHOD_SIMPLE_TYPE_GTC]             = { "gtc",      TRUE  },
+	[EAP_METHOD_SIMPLE_TYPE_UNKNOWN]         = { "unknown",  TRUE  },
 };
 
 static void
-fill_connection (EAPMethod *parent, NMConnection *connection, NMSettingSecretFlags prev_flags)
+fill_connection (EAPMethod *parent, NMConnection *connection)
 {
 	EAPMethodSimple *method = (EAPMethodSimple *) parent;
 	NMSetting8021x *s_8021x;
@@ -136,51 +182,64 @@ fill_connection (EAPMethod *parent, NMConnection *connection, NMSettingSecretFla
 	s_8021x = nm_connection_get_setting_802_1x (connection);
 	g_assert (s_8021x);
 
-	/* If this is the main EAP method, clear any existing methods because the
-	 * user-selected on will replace it.
-	 */
-	if (parent->phase2 == FALSE)
-		nm_setting_802_1x_clear_eap_methods (s_8021x);
-
-	eap_type = &eap_table[method->type];
-	if (parent->phase2) {
-		/* If the outer EAP method (TLS, TTLS, PEAP, etc) allows inner/phase2
-		 * EAP methods (which only TTLS allows) *and* the inner/phase2 method
-		 * supports being an inner EAP method, then set PHASE2_AUTHEAP.
-		 * Otherwise the inner/phase2 method goes into PHASE2_AUTH.
-		 */
-		if ((method->flags & EAP_METHOD_SIMPLE_FLAG_AUTHEAP_ALLOWED) && eap_type->autheap_allowed) {
-			g_object_set (s_8021x, NM_SETTING_802_1X_PHASE2_AUTHEAP, eap_type->name, NULL);
-			g_object_set (s_8021x, NM_SETTING_802_1X_PHASE2_AUTH, NULL, NULL);
-		} else {
-			g_object_set (s_8021x, NM_SETTING_802_1X_PHASE2_AUTH, eap_type->name, NULL);
-			g_object_set (s_8021x, NM_SETTING_802_1X_PHASE2_AUTHEAP, NULL, NULL);
-		}
-	} else
-		nm_setting_802_1x_add_eap_method (s_8021x, eap_type->name);
-
-	g_object_set (s_8021x, NM_SETTING_802_1X_IDENTITY, gtk_entry_get_text (method->username_entry), NULL);
-
-	/* Save the password always ask setting */
-	not_saved = always_ask_selected (method->password_entry);
-	flags = nma_utils_menu_to_secret_flags (GTK_WIDGET (method->password_entry));
-	nm_setting_set_secret_flags (NM_SETTING (s_8021x), NM_SETTING_802_1X_PASSWORD, flags, NULL);
-
-	/* Fill the connection's password if we're in the applet so that it'll get
-	 * back to NM.  From the editor though, since the connection isn't going
-	 * back to NM in response to a GetSecrets() call, we don't save it if the
-	 * user checked "Always Ask".
-	 */
-	if (!(method->flags & EAP_METHOD_SIMPLE_FLAG_IS_EDITOR) || not_saved == FALSE)
-		g_object_set (s_8021x, NM_SETTING_802_1X_PASSWORD, gtk_entry_get_text (method->password_entry), NULL);
-
-	/* Update secret flags and popup when editing the connection */
 	if (!(method->flags & EAP_METHOD_SIMPLE_FLAG_SECRETS_ONLY)) {
-		GtkWidget *passwd_entry = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_password_entry"));
-		g_assert (passwd_entry);
+		/* If this is the main EAP method, clear any existing methods because the
+		 * user-selected one will replace it.
+		 */
+		if (parent->phase2 == FALSE)
+			nm_setting_802_1x_clear_eap_methods (s_8021x);
 
-		nma_utils_update_password_storage (passwd_entry, flags,
-		                                   NM_SETTING (s_8021x), parent->password_flags_name);
+		eap_type = &eap_table[method->type];
+		if (parent->phase2) {
+			/* If the outer EAP method (TLS, TTLS, PEAP, etc) allows inner/phase2
+			 * EAP methods (which only TTLS allows) *and* the inner/phase2 method
+			 * supports being an inner EAP method, then set PHASE2_AUTHEAP.
+			 * Otherwise the inner/phase2 method goes into PHASE2_AUTH.
+			 */
+			if ((method->flags & EAP_METHOD_SIMPLE_FLAG_AUTHEAP_ALLOWED) && eap_type->autheap_allowed) {
+				g_object_set (s_8021x, NM_SETTING_802_1X_PHASE2_AUTHEAP, eap_type->name, NULL);
+				g_object_set (s_8021x, NM_SETTING_802_1X_PHASE2_AUTH, NULL, NULL);
+			} else {
+				g_object_set (s_8021x, NM_SETTING_802_1X_PHASE2_AUTH, eap_type->name, NULL);
+				g_object_set (s_8021x, NM_SETTING_802_1X_PHASE2_AUTHEAP, NULL, NULL);
+			}
+		} else
+			nm_setting_802_1x_add_eap_method (s_8021x, eap_type->name);
+	}
+
+	if (method->username_requested)
+		g_object_set (s_8021x, NM_SETTING_802_1X_IDENTITY, gtk_entry_get_text (method->username_entry), NULL);
+
+	if (method->password_requested) {
+		/* Save the password always ask setting */
+		not_saved = always_ask_selected (method->password_entry);
+		flags = nma_utils_menu_to_secret_flags (GTK_WIDGET (method->password_entry));
+		nm_setting_set_secret_flags (NM_SETTING (s_8021x), method->password_flags_name, flags, NULL);
+
+		/* Fill the connection's password if we're in the applet so that it'll get
+		 * back to NM.  From the editor though, since the connection isn't going
+		 * back to NM in response to a GetSecrets() call, we don't save it if the
+		 * user checked "Always Ask".
+		 */
+		if (!(method->flags & EAP_METHOD_SIMPLE_FLAG_IS_EDITOR) || not_saved == FALSE) {
+			g_object_set (s_8021x, NM_SETTING_802_1X_PASSWORD,
+			              gtk_entry_get_text (method->password_entry), NULL);
+		}
+
+		/* Update secret flags and popup when editing the connection */
+		if (!(method->flags & EAP_METHOD_SIMPLE_FLAG_SECRETS_ONLY)) {
+			GtkWidget *passwd_entry = GTK_WIDGET (gtk_builder_get_object (parent->builder,
+			                                                              "eap_simple_password_entry"));
+			g_assert (passwd_entry);
+
+			nma_utils_update_password_storage (passwd_entry, flags,
+			                                   NM_SETTING (s_8021x), method->password_flags_name);
+		}
+	}
+
+	if (method->pkey_passphrase_requested) {
+		g_object_set (s_8021x, NM_SETTING_802_1X_PRIVATE_KEY_PASSWORD,
+		              gtk_entry_get_text (method->pkey_passphrase_entry), NULL);
 	}
 }
 
@@ -192,6 +251,11 @@ update_secrets (EAPMethod *parent, NMConnection *connection)
 	                          "eap_simple_password_entry",
 	                          NM_TYPE_SETTING_802_1X,
 	                          (HelperSecretFunc) nm_setting_802_1x_get_password);
+	helper_fill_secret_entry (connection,
+	                          parent->builder,
+	                          "eap_simple_pkey_passphrase_entry",
+	                          NM_TYPE_SETTING_802_1X,
+	                          (HelperSecretFunc) nm_setting_802_1x_get_private_key_password);
 }
 
 static gboolean
@@ -268,37 +332,37 @@ destroy (EAPMethod *parent)
 
 	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_notebook"));
 	g_assert (widget);
+	g_signal_handlers_disconnect_by_data (widget, method);
 
-	g_signal_handlers_disconnect_by_func (G_OBJECT (widget),
-	                                      (GCallback) widgets_realized,
-	                                      method);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (widget),
-	                                      (GCallback) widgets_unrealized,
-	                                      method);
+	g_signal_handlers_disconnect_by_data (method->username_entry, method->ws_parent);
+	g_signal_handlers_disconnect_by_data (method->password_entry, method->ws_parent);
+	g_signal_handlers_disconnect_by_data (method->password_entry, method);
+	g_signal_handlers_disconnect_by_data (method->show_password, method);
+	g_signal_handlers_disconnect_by_data (method->pkey_passphrase_entry, method->ws_parent);
+	g_signal_handlers_disconnect_by_data (method->show_pkey_passphrase, method);
 
-	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_password_entry"));
-	g_assert (widget);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (widget),
-	                                      (GCallback) password_storage_changed,
-	                                      method);
-	if (method->idle_func_id > 0) {
-		g_source_remove (method->idle_func_id);
-		method->idle_func_id = 0;
-	}
+	nm_clear_g_source (&method->idle_func_id);
+}
 
-	wireless_security_unref (method->ws_parent);
+static void
+hide_row (GtkWidget **widgets, size_t num)
+{
+	while (num--)
+		gtk_widget_hide (*widgets++);
 }
 
 EAPMethodSimple *
 eap_method_simple_new (WirelessSecurity *ws_parent,
                        NMConnection *connection,
                        EAPMethodSimpleType type,
-                       EAPMethodSimpleFlags flags)
+                       EAPMethodSimpleFlags flags,
+                       const char *const*hints)
 {
 	EAPMethod *parent;
 	EAPMethodSimple *method;
 	GtkWidget *widget;
 	NMSetting8021x *s_8021x = NULL;
+	GtkWidget *widget_row[10];
 
 	parent = eap_method_init (sizeof (EAPMethodSimple),
 	                          validate,
@@ -306,19 +370,36 @@ eap_method_simple_new (WirelessSecurity *ws_parent,
 	                          fill_connection,
 	                          update_secrets,
 	                          destroy,
-	                          UIDIR "/eap-method-simple.ui",
+	                          "/org/freedesktop/network-manager-applet/eap-method-simple.ui",
 	                          "eap_simple_notebook",
 	                          "eap_simple_username_entry",
 	                          flags & EAP_METHOD_SIMPLE_FLAG_PHASE2);
 	if (!parent)
 		return NULL;
 
-	parent->password_flags_name = NM_SETTING_802_1X_PASSWORD;
 	method = (EAPMethodSimple *) parent;
-	method->ws_parent = wireless_security_ref (ws_parent);
+	method->password_flags_name = NM_SETTING_802_1X_PASSWORD;
+	method->ws_parent = ws_parent;
 	method->flags = flags;
 	method->type = type;
 	g_assert (type < EAP_METHOD_SIMPLE_TYPE_LAST);
+	g_assert (   type != EAP_METHOD_SIMPLE_TYPE_UNKNOWN
+	          || hints);
+
+	if (hints) {
+		for (; *hints; hints++) {
+			if (!strcmp (*hints, NM_SETTING_802_1X_IDENTITY))
+				method->username_requested = TRUE;
+			else if (!strcmp (*hints, NM_SETTING_802_1X_PASSWORD)) {
+				method->password_requested = TRUE;
+				method->password_flags_name = NM_SETTING_802_1X_PASSWORD;
+			} else if (!strcmp (*hints, NM_SETTING_802_1X_PRIVATE_KEY_PASSWORD))
+				method->pkey_passphrase_requested = TRUE;
+		}
+	} else {
+		method->username_requested = TRUE;
+		method->password_requested = TRUE;
+	}
 
 	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_notebook"));
 	g_assert (widget);
@@ -336,7 +417,8 @@ eap_method_simple_new (WirelessSecurity *ws_parent,
 	                  (GCallback) wireless_security_changed_cb,
 	                  ws_parent);
 
-	if (method->flags & EAP_METHOD_SIMPLE_FLAG_SECRETS_ONLY)
+	if (   (method->flags & EAP_METHOD_SIMPLE_FLAG_SECRETS_ONLY)
+	    && !method->username_requested)
 		gtk_widget_set_sensitive (widget, FALSE);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_password_entry"));
@@ -349,7 +431,7 @@ eap_method_simple_new (WirelessSecurity *ws_parent,
 	/* Create password-storage popup menu for password entry under entry's secondary icon */
 	if (connection)
 		s_8021x = nm_connection_get_setting_802_1x (connection);
-	nma_utils_setup_password_storage (widget, 0, (NMSetting *) s_8021x, parent->password_flags_name,
+	nma_utils_setup_password_storage (widget, 0, (NMSetting *) s_8021x, method->password_flags_name,
 	                                  FALSE, flags & EAP_METHOD_SIMPLE_FLAG_SECRETS_ONLY);
 
 	g_signal_connect (method->password_entry, "notify::secondary-icon-name",
@@ -360,9 +442,40 @@ eap_method_simple_new (WirelessSecurity *ws_parent,
 	g_assert (widget);
 	method->show_password = GTK_TOGGLE_BUTTON (widget);
 	g_signal_connect (G_OBJECT (widget), "toggled",
-	                  (GCallback) show_toggled_cb,
+	                  (GCallback) show_password_toggled_cb,
 	                  method);
 
+	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_pkey_passphrase_entry"));
+	g_assert (widget);
+	method->pkey_passphrase_entry = GTK_ENTRY (widget);
+	g_signal_connect (G_OBJECT (widget), "changed",
+	                  (GCallback) wireless_security_changed_cb,
+	                  ws_parent);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_show_pkey_passphrase_checkbutton"));
+	g_assert (widget);
+	method->show_pkey_passphrase = GTK_TOGGLE_BUTTON (widget);
+	g_signal_connect (G_OBJECT (widget), "toggled",
+	                  (GCallback) show_pkey_passphrase_toggled_cb,
+	                  method);
+
+	widget_row[0] = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_username_label"));
+	widget_row[1] = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_username_entry"));
+	if (!method->username_requested)
+		hide_row (widget_row, 2);
+
+	widget_row[0] = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_password_label"));
+	widget_row[1] = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_password_entry"));
+	widget_row[2] = GTK_WIDGET (gtk_builder_get_object (parent->builder, "show_checkbutton_eapsimple"));
+	if (!method->password_requested)
+		hide_row (widget_row, 3);
+
+	widget_row[0] = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_pkey_passphrase_label"));
+	widget_row[1] = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_pkey_passphrase_entry"));
+	widget_row[2] = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_show_pkey_passphrase_checkbutton"));
+	if (!method->pkey_passphrase_requested)
+		hide_row (widget_row, 3);
+
 	/* Initialize the UI fields with the security settings from method->ws_parent.
 	 * This will be done again when the widget gets realized. It must be done here as well,
 	 * because the outer dialog will ask to 'validate' the connection before the security tab
@@ -373,4 +486,3 @@ eap_method_simple_new (WirelessSecurity *ws_parent,
 
 	return method;
 }
-
